@@ -4,7 +4,7 @@ import os.path as op
 
 import click
 
-from .base import lgr, map_to_click_exceptions
+from .base import devel_option, lgr, map_to_click_exceptions
 from ..dandiarchive import DandisetURL, _dandi_url_parser, parse_dandi_url
 from ..utils import is_url
 
@@ -33,7 +33,7 @@ List .nwb files and dandisets metadata.
     "--format",
     help="Choose the format/frontend for output. If 'auto', 'pyout' will be "
     "used in case of multiple files, and 'yaml' for a single file.",
-    type=click.Choice(["auto", "pyout", "json", "json_pp", "yaml"]),
+    type=click.Choice(["auto", "pyout", "json", "json_pp", "json_lines", "yaml"]),
     default="auto",
 )
 @click.option(
@@ -60,13 +60,32 @@ List .nwb files and dandisets metadata.
     help="Convert metadata to new schema version",
     metavar="VERSION",
 )
+@devel_option(
+    "--use-fake-digest",
+    is_flag=True,
+    help="Use dummy value for digests of local files instead of computing",
+)
 @click.argument("paths", nargs=-1, type=click.Path(exists=False, dir_okay=True))
 @map_to_click_exceptions
-def ls(paths, schema, metadata, fields=None, format="auto", recursive=False, jobs=6):
-    """ List .nwb files and dandisets metadata. """
+def ls(
+    paths,
+    schema,
+    metadata,
+    use_fake_digest=False,
+    fields=None,
+    format="auto",
+    recursive=False,
+    jobs=6,
+):
+    """List .nwb files and dandisets metadata."""
 
     # TODO: more logical ordering in case of fields = None
-    from .formatter import JSONFormatter, PYOUTFormatter, YAMLFormatter
+    from .formatter import (
+        JSONFormatter,
+        JSONLinesFormatter,
+        PYOUTFormatter,
+        YAMLFormatter,
+    )
     from ..consts import metadata_all_fields
 
     # TODO: avoid
@@ -75,13 +94,13 @@ def ls(paths, schema, metadata, fields=None, format="auto", recursive=False, job
 
     common_fields = ("path", "size")
     if schema is not None:
-        from ..models import AssetMeta, DandisetMeta
+        from dandischema import models
 
         all_fields = tuple(
             sorted(
                 set(common_fields)
-                | DandisetMeta.__fields__.keys()
-                | AssetMeta.__fields__.keys()
+                | models.Dandiset.__fields__.keys()
+                | models.Asset.__fields__.keys()
             )
         )
     else:
@@ -111,21 +130,20 @@ def ls(paths, schema, metadata, fields=None, format="auto", recursive=False, job
         for path in paths:
             if is_url(path):
                 parsed_url = parse_dandi_url(path)
-                with parsed_url.navigate(
-                    include_metadata=metadata in ("all", "assets")
-                ) as (client, dandiset, assets):
+                with parsed_url.navigate() as (client, dandiset, assets):
                     if isinstance(parsed_url, DandisetURL):
                         rec = {
-                            "path": dandiset.pop("dandiset", {}).get(
-                                "identifier", "ERR#%s" % id(dandiset)
-                            )
+                            "path": dandiset.identifier,
+                            **dandiset.version.json_dict(),
+                            "metadata": dandiset.get_raw_metadata(),
                         }
-                        # flatten the metadata into record to display
-                        # rec.update(dandiset.get('metadata', {}))
-                        rec.update(dandiset)
                         yield rec
                     if not isinstance(parsed_url, DandisetURL) or recursive:
-                        yield from assets
+                        for a in assets:
+                            rec = a.json_dict()
+                            if metadata in ("all", "assets"):
+                                rec["metadata"] = a.get_raw_metadata()
+                            yield rec
             else:
                 # For now we support only individual files
                 yield path
@@ -144,6 +162,8 @@ def ls(paths, schema, metadata, fields=None, format="auto", recursive=False, job
         out = JSONFormatter()
     elif format == "json_pp":
         out = JSONFormatter(indent=2)
+    elif format == "json_lines":
+        out = JSONLinesFormatter()
     elif format == "yaml":
         out = YAMLFormatter()
     else:
@@ -172,6 +192,7 @@ def ls(paths, schema, metadata, fields=None, format="auto", recursive=False, job
                             errors=errors,
                             flatten=format == "pyout",
                             schema=schema,
+                            use_fake_digest=use_fake_digest,
                         )
                         if format == "pyout":
                             rec[async_keys] = cb
@@ -300,10 +321,13 @@ def flatten_meta_to_pyout(meta):
     return out
 
 
-def get_metadata_ls(path, keys, errors, flatten=False, schema=None):
+def get_metadata_ls(
+    path, keys, errors, flatten=False, schema=None, use_fake_digest=False
+):
     from ..dandiset import APIDandiset
     from ..metadata import get_metadata, nwb2asset
     from ..pynwb_utils import get_nwb_version, ignore_benign_pynwb_warnings
+    from ..support.digests import get_digest
 
     ignore_benign_pynwb_warnings()
 
@@ -317,7 +341,17 @@ def get_metadata_ls(path, keys, errors, flatten=False, schema=None):
                         dandiset = APIDandiset(path, schema_version=schema)
                         rec = dandiset.metadata
                     else:
-                        rec = nwb2asset(path, schema_version=schema).json_dict()
+                        if use_fake_digest:
+                            digest = "0" * 32 + "-1"
+                        else:
+                            lgr.info("Calculating digest for %s", path)
+                            digest = get_digest(path, digest="dandi-etag")
+                        rec = nwb2asset(
+                            path,
+                            schema_version=schema,
+                            digest=digest,
+                            digest_type="dandi_etag",
+                        ).json_dict()
                 else:
                     rec = get_metadata(path)
             except Exception as exc:
